/* ccgo: igs/match_box.cc
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "match_box.hh"
#include <gtkmm/label.h>
#include <gtkmm/button.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/buttonbox.h>
#include <gtkmm/stock.h>
#include <sstream>

using namespace igs;

void MatchBox::send_match()
{
	match.my_turn = my_turn.get_history() == 0 ? go::TURN_BLACK : go::TURN_WHITE;
	match.board_size = int(board_size->get_value());
	match.time = int(time->get_value());
	match.byo_time = int (byo_time->get_value());
	match.set_option = set_option.get_active();
	match_request(match);
	head_line.set_text("Match request sent");
}

void MatchBox::set_option_toggle()
{
	if (set_option.get_active()) {
		match.set_option = true;
		option_box.set_sensitive(true);
	} else {
		match.set_option = false;
		option_box.set_sensitive(false);
	}
}

void MatchBox::decline_match()
{
	match.decline = true;
	match_request(match);
	match_cancel(this); // close window after declining the match
}

MatchBox::MatchBox(const std::string & n) :
	Gtk::Table(2),
	set_option("Set details", false),
	board_size(Gtk::manage(new Gtk::Adjustment(19, 5, 23))),
	time(Gtk::manage(new Gtk::Adjustment(75, 0, 530))),
	byo_time(Gtk::manage(new Gtk::Adjustment(0, 0, 900))),
	decline_button(Gtk::Stock::NO)
{
	Gtk::VBox * vb;
	vb = Gtk::manage(new Gtk::VBox);
	attach(* vb, 0, 1, 0, 1);

	vb->pack_start(head_line, Gtk::PACK_SHRINK);
	head_line.set_text(std::string("Match with ") + n);
	match.opponent = n;

	vb->pack_start(set_option, Gtk::PACK_SHRINK);
	set_option.signal_toggled().connect(SigC::slot(* this, & MatchBox::set_option_toggle));

	vb->pack_start(option_box, Gtk::PACK_SHRINK);

	Gtk::HBox * hb;
	// my turn
	hb = Gtk::manage(new Gtk::HBox);
	option_box.pack_start(* hb, Gtk::PACK_SHRINK);
	hb->pack_start(* Gtk::manage(new Gtk::Label("My turn: ")), Gtk::PACK_SHRINK);
	hb->pack_start(my_turn, Gtk::PACK_SHRINK);
	Gtk::Menu * m = Gtk::manage(new Gtk::Menu);
	m->append(* Gtk::manage(new Gtk::MenuItem("Black")));
	m->append(* Gtk::manage(new Gtk::MenuItem("White")));
	my_turn.set_menu(* m);
	my_turn.set_history(0);
	// board size
	hb = Gtk::manage(new Gtk::HBox);
	option_box.pack_start(* hb, Gtk::PACK_SHRINK);
	Gtk::SpinButton * sb;
	hb->pack_start(* Gtk::manage(new Gtk::Label("Board size: ")), Gtk::PACK_SHRINK);
	sb = Gtk::manage(new Gtk::SpinButton(* board_size));
	hb->pack_start(* sb, Gtk::PACK_SHRINK);
	// main time
	hb = Gtk::manage(new Gtk::HBox);
	option_box.pack_start(* hb, Gtk::PACK_SHRINK);
	hb->pack_start(* Gtk::manage(new Gtk::Label("Main time: ")), Gtk::PACK_SHRINK);
	sb = Gtk::manage(new Gtk::SpinButton(* time));
	hb->pack_start(* sb, Gtk::PACK_SHRINK);
	hb->pack_start(* Gtk::manage(new Gtk::Label(" min.")), Gtk::PACK_SHRINK);
	// byo time
	hb = Gtk::manage(new Gtk::HBox);
	option_box.pack_start(* hb, Gtk::PACK_SHRINK);
	hb->pack_start(* Gtk::manage(new Gtk::Label("Byo-yomi time: ")), Gtk::PACK_SHRINK);
	sb = Gtk::manage(new Gtk::SpinButton(* byo_time));
	hb->pack_start(* sb, Gtk::PACK_SHRINK);
	hb->pack_start(* Gtk::manage(new Gtk::Label(" min.")), Gtk::PACK_SHRINK);

	attach(button_area, 0, 1, 1, 2, Gtk::FILL);
	Gtk::Button * bt;

	bt = Gtk::manage(new Gtk::Button(Gtk::Stock::OK));
	button_area.pack_end(* bt);
	bt->signal_clicked().connect(SigC::slot(* this, & MatchBox::send_match));

	button_area.pack_end(decline_button);
	decline_button.signal_clicked().connect(SigC::slot(* this, & MatchBox::decline_match));

	bt = Gtk::manage(new Gtk::Button(Gtk::Stock::CANCEL));
	button_area.pack_end(* bt);
	bt->signal_clicked().connect(SigC::bind(match_cancel.slot(), this));

	show_all_children();
	option_box.set_sensitive(false);
	decline_button.hide();
	show();
}

MatchBox::~MatchBox()
{
// 	std::cerr << "deleting MatchBox" << endl;
}

void MatchBox::update(const Match & m) // incoming match request
{
	if (m.get_automatch()) {
		std::ostringstream o;
		o << "Automatch from " << m.opponent << "(B.Y. Stones:" << m.byo_stone << ")";
		head_line.set_text(o.str());
		match.set_automatch(true);
		set_option.set_active(false);
	} else {
		head_line.set_text(std::string("Match request from ") + m.opponent);
		match.set_automatch(false);
	}
	if (m.set_option) {
		if (m.my_turn == go::TURN_BLACK) my_turn.set_history(0);
		else my_turn.set_history(1);
		board_size->set_value(m.board_size);
		time->set_value(m.time);
		byo_time->set_value(m.byo_time);
		set_option.set_active(true);
	}
	decline_button.show(); // I can decline an incoming match
}

void MatchBox::decline()
{
	head_line.set_text(match.opponent + " declined the match");
}

void MatchBox::not_online()
{
	head_line.set_text(match.opponent + " is offline");
}

void MatchBox::not_open()
{
	head_line.set_text(match.opponent + " is not accepting match");
}

void MatchBox::in_match()
{
	head_line.set_text(match.opponent + " is already in a match");
}
